==> derquinsej-hib3-0.0.10-sources.jar: net/sf/derquinsej/hib3/GenericDAOImpl.java <==
/*
 * Copyright 2007 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.sf.derquinsej.hib3;

import static com.google.common.base.Preconditions.checkNotNull;

import java.io.Serializable;
import java.lang.reflect.ParameterizedType;
import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.LockMode;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.criterion.Criterion;

import com.google.common.base.Preconditions;

/**
 * Implementation of the Generic DAO.
 * @author Andres Rodriguez
 */
public abstract class GenericDAOImpl<T, ID extends Serializable> implements GenericDAO<T, ID> {
	/** Persistent class. */
	private Class<T> persistentClass;
	/** Hibernate Session factory. */
	private final SessionFactory sessionFactory;

	private static SessionFactory check(SessionFactory sessionFactory) {
		return checkNotNull(sessionFactory, "A session factory must be provided.");
	}

	/**
	 * Constructs the DAO
	 * @param sessionFactory Hibernate Session factory.
	 */
	public GenericDAOImpl(final SessionFactory sessionFactory) {
		@SuppressWarnings("unchecked")
		final Class<T> type = (Class<T>) ((ParameterizedType) getClass().getGenericSuperclass())
				.getActualTypeArguments()[0];
		this.persistentClass = type;
		this.sessionFactory = check(sessionFactory);
	}

	/**
	 * Constructs the DAO.
	 * @param persistentClass Persistent class.
	 * @param sessionFactory Hibernate Session factory.
	 */
	public GenericDAOImpl(final Class<T> persistentClass, final SessionFactory sessionFactory) {
		this.persistentClass = checkNotNull(persistentClass);
		this.sessionFactory = check(sessionFactory);
	}

	/**
	 * Returns the current session.
	 * @return The current session.
	 */
	protected final Session getSession() {
		return sessionFactory.getCurrentSession();
	}

	/**
	 * Returns the persistent class.
	 * @return The persistent class.
	 */
	protected final Class<T> getPersistentClass() {
		return persistentClass;
	}

	/*
	 * (non-Javadoc)
	 * @see net.sf.derquinsej.hib3.GenericDAO#clear()
	 */
	public void clear() {
		getSession().clear();
	}

	/*
	 * (non-Javadoc)
	 * @see net.sf.derquinsej.hib3.GenericDAO#delete(java.lang.Object)
	 */
	public void delete(T entity) {
		Preconditions.checkNotNull(entity);
		getSession().delete(entity);
	}

	/*
	 * (non-Javadoc)
	 * @see net.sf.derquinsej.hib3.GenericDAO#deleteById(java.io.Serializable)
	 */
	public void deleteById(ID id) {
		Preconditions.checkNotNull(id);
		T entity = findById(id, true);
		if (entity != null) {
			delete(entity);
		}
	}

	/*
	 * (non-Javadoc)
	 * @see net.sf.derquinsej.hib3.GenericDAO#findAll()
	 */
	public List<T> findAll() {
		return findByCriteria();
	}

	/*
	 * (non-Javadoc)
	 * @see net.sf.derquinsej.hib3.GenericDAO#findById(java.io.Serializable, boolean)
	 */
	@SuppressWarnings("unchecked")
	public T findById(ID id, boolean lock) {
		if (lock) {
			return (T)getSession().get(persistentClass, id, LockMode.UPGRADE);
		}
		return (T)getSession().get(persistentClass, id);
	}

	/*
	 * (non-Javadoc)
	 * @see net.sf.derquinsej.hib3.GenericDAO#flush()
	 */
	public void flush() {
		getSession().flush();

	}

	/*
	 * (non-Javadoc)
	 * @see net.sf.derquinsej.hib3.GenericDAO#save(java.lang.Object)
	 */
	public T save(T entity) {
		Preconditions.checkNotNull(entity);
		getSession().save(entity);
		return entity;
	}

	/*
	 * (non-Javadoc)
	 * @see net.sf.derquinsej.hib3.GenericDAO#saveOrUpdate(java.lang.Object)
	 */
	public T saveOrUpdate(T entity) {
		Preconditions.checkNotNull(entity);
		getSession().saveOrUpdate(entity);
		return entity;
	}

	/*
	 * (non-Javadoc)
	 * @see net.sf.derquinsej.hib3.GenericDAO#update(java.lang.Object)
	 */
	public T update(T entity) {
		Preconditions.checkNotNull(entity);
		getSession().update(entity);
		return entity;
	}

	/**
	 * Returns a new criteria object.
	 * @return A new criteria object.
	 */
	protected final Criteria newCriteria() {
		return getSession().createCriteria(persistentClass);
	}

	/**
	 * Returns the list of entity instances that matches the provided crtieria.
	 * @param criterion Search criteria.
	 * @return The results.
	 */
	protected final List<T> findByCriteria(Criterion... criterion) {
		final Criteria criteria = newCriteria();
		for (Criterion c : criterion) {
			criteria.add(c);
		}
		@SuppressWarnings("unchecked")
		final List<T> list = criteria.list();
		return list;
	}

}

==> derquinsej-hib3-0.0.11-sources.jar: net/sf/derquinsej/hib3/GenericDAOImpl.java <==
/*
 * Copyright 2007 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.sf.derquinsej.hib3;

import static com.google.common.base.Preconditions.checkNotNull;

import java.io.Serializable;
import java.lang.reflect.ParameterizedType;
import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.LockMode;
import org.hibernate.NonUniqueResultException;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.criterion.Criterion;

import com.google.common.base.Preconditions;

/**
 * Implementation of the Generic DAO.
 * @author Andres Rodriguez
 */
public abstract class GenericDAOImpl<T, ID extends Serializable> implements GenericDAO<T, ID> {
	/** Persistent class. */
	private Class<T> persistentClass;
	/** Hibernate Session factory. */
	private final SessionFactory sessionFactory;

	private static SessionFactory check(SessionFactory sessionFactory) {
		return checkNotNull(sessionFactory, "A session factory must be provided.");
	}

	/**
	 * Constructs the DAO
	 * @param sessionFactory Hibernate Session factory.
	 */
	public GenericDAOImpl(final SessionFactory sessionFactory) {
		@SuppressWarnings("unchecked")
		final Class<T> type = (Class<T>) ((ParameterizedType) getClass().getGenericSuperclass())
				.getActualTypeArguments()[0];
		this.persistentClass = type;
		this.sessionFactory = check(sessionFactory);
	}

	/**
	 * Constructs the DAO.
	 * @param persistentClass Persistent class.
	 * @param sessionFactory Hibernate Session factory.
	 */
	public GenericDAOImpl(final Class<T> persistentClass, final SessionFactory sessionFactory) {
		this.persistentClass = checkNotNull(persistentClass);
		this.sessionFactory = check(sessionFactory);
	}

	/**
	 * Returns the current session.
	 * @return The current session.
	 */
	protected final Session getSession() {
		return sessionFactory.getCurrentSession();
	}

	/**
	 * Returns the persistent class.
	 * @return The persistent class.
	 */
	protected final Class<T> getPersistentClass() {
		return persistentClass;
	}

	/*
	 * (non-Javadoc)
	 * @see net.sf.derquinsej.hib3.GenericDAO#clear()
	 */
	public void clear() {
		getSession().clear();
	}

	/*
	 * (non-Javadoc)
	 * @see net.sf.derquinsej.hib3.GenericDAO#delete(java.lang.Object)
	 */
	public void delete(T entity) {
		Preconditions.checkNotNull(entity);
		getSession().delete(entity);
	}

	/*
	 * (non-Javadoc)
	 * @see net.sf.derquinsej.hib3.GenericDAO#deleteById(java.io.Serializable)
	 */
	public void deleteById(ID id) {
		Preconditions.checkNotNull(id);
		T entity = findById(id, true);
		if (entity != null) {
			delete(entity);
		}
	}

	/*
	 * (non-Javadoc)
	 * @see net.sf.derquinsej.hib3.GenericDAO#findAll()
	 */
	public List<T> findAll() {
		return findByCriteria();
	}

	/*
	 * (non-Javadoc)
	 * @see net.sf.derquinsej.hib3.GenericDAO#findById(java.io.Serializable, boolean)
	 */
	@SuppressWarnings("unchecked")
	public T findById(ID id, boolean lock) {
		if (lock) {
			return (T)getSession().get(persistentClass, id, LockMode.UPGRADE);
		}
		return (T)getSession().get(persistentClass, id);
	}

	/*
	 * (non-Javadoc)
	 * @see net.sf.derquinsej.hib3.GenericDAO#flush()
	 */
	public void flush() {
		getSession().flush();

	}

	/*
	 * (non-Javadoc)
	 * @see net.sf.derquinsej.hib3.GenericDAO#save(java.lang.Object)
	 */
	public T save(T entity) {
		Preconditions.checkNotNull(entity);
		getSession().save(entity);
		return entity;
	}

	/*
	 * (non-Javadoc)
	 * @see net.sf.derquinsej.hib3.GenericDAO#saveOrUpdate(java.lang.Object)
	 */
	public T saveOrUpdate(T entity) {
		Preconditions.checkNotNull(entity);
		getSession().saveOrUpdate(entity);
		return entity;
	}

	/*
	 * (non-Javadoc)
	 * @see net.sf.derquinsej.hib3.GenericDAO#update(java.lang.Object)
	 */
	public T update(T entity) {
		Preconditions.checkNotNull(entity);
		getSession().update(entity);
		return entity;
	}

	/**
	 * Returns a new criteria object.
	 * @return A new criteria object.
	 */
	protected final Criteria newCriteria() {
		return getSession().createCriteria(persistentClass);
	}

	/**
	 * Returns the list of entity instances that matches the provided crtieria.
	 * @param criterion Search criteria.
	 * @return The results.
	 */
	protected final List<T> findByCriteria(Criterion... criterion) {
		final Criteria criteria = newCriteria();
		for (Criterion c : criterion) {
			criteria.add(c);
		}
		@SuppressWarnings("unchecked")
		final List<T> list = criteria.list();
		return list;
	}
	
	/**
	 * Returns a externally defined named query.
	 * @param queryName Query name.
	 * @return The requested query.
	 */
	protected final Query getNamedQuery(String queryName) {
		Preconditions.checkNotNull(queryName);
		return getSession().getNamedQuery(queryName);
	}

	/**
	 * Returns the results of a query as a list of the persisted class.
	 * @param query Query to execute.
	 * @return The results.
	 */
	protected final List<T> list(Query query) {
		Preconditions.checkNotNull(query);
		@SuppressWarnings("unchecked")
		final List<T> list = query.list();
		return list;
	}

	/**
	 * Returns the first result of a query returning instances of the persisted class.
	 * @param query Query to execute.
	 * @return The first result or {@code null} if there are no results.
	 */
	protected final T first(Query query) {
		final List<T> list = list(query);
		if (list != null && !list.isEmpty()) {
			return list.get(0);
		}
		return null;
	}

	/**
	 * Returns the unique result of a query returning instances of the persisted class.
	 * @param query Query to execute.
	 * @return The unique result or {@code null} if there are no results.
	 * @throws NonUniqueResultException if there are more than one result.
	 */
	protected final T unique(Query query) {
		Preconditions.checkNotNull(query);
		@SuppressWarnings("unchecked")
		final T result = (T)query.uniqueResult();
		return result;
	}
	
}
